<?php
/* vim: set expandtab tabstop=4 softtabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2002 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Author: Geir Torstein Kristiansen <gtk@techie.online.no>             |
// +----------------------------------------------------------------------+
//

// for converting colorcoded text to various formats using correct palette 

class NSCO_Color
{
    // base quake3 console color palette.

    var $_BASE_Colors = array('0' => '000000', // black
                              '1' => 'FF0000', // red
                              '2' => '00FF00', // green
                              '3' => 'FFFF00', // yellow
                              '4' => '0000FF', // blue
                              '5' => '00FFFF', // cyan 
                              '6' => 'FF00FF', // magenta
                              '7' => 'FFFFFF'  // white
                             );

    // mIRC color palette. The irc client background color may be either white 
    // or black so we redefine them to gray and light grey as we cannot know.

    var $_MIRC_Colors = array('FFFFFF' => '15', // white
                              '000000' => '14', // black
                              '00007B' => '02', // dark blue
                              '009200' => '03', // dark green
                              'FF0000' => '04', // red
                              '7B0000' => '05', // brown
                              '9C009C' => '06', // purple
                              'FF7D00' => '07', // orange
                              'FFFF00' => '08', // yellow
                              '00FF00' => '09', // green 
                              '009294' => '10', // dark cyan
                              '00FFFF' => '11', // cyan 
                              '0000FF' => '12', // blue
                              'FF00FF' => '13', // magenta
                              '7B7D7B' => '14', // grey
                              'D6D3D6' => '15'  // light grey
                             );
    
    // NS:CO console color palette thanks to iCu|Gate.

    var $_NSCO_Colors = array('a' => 'BA0000',
                              'b' => '3242A2',
                              'c' => '2B1351',
                              'd' => 'AC7938',
                              'e' => '8A3B8A',
                              'f' => '75809C',
                              'g' => '780043',
                              'h' => '7EAEB8',
                              'i' => '82BF3D',
                              'j' => '5E5254',
                              'k' => '5EA7B8',
                              'l' => '68B742',
                              'm' => 'AF998C',
                              'n' => '83A4B7',
                              'o' => '872987',
                              'p' => '526E3E',
                              'q' => '8D716E',
                              'r' => '3B7F6A',
                              's' => '756767',
                              't' => '9D5795',
                              'u' => '2D0162',
                              'v' => '390F1B',
                              'w' => '2C1A70',
                              'x' => '11A849',
                              'y' => '000000',
                              'z' => '020814',
                              'A' => '6C9553',
                              'B' => 'A5823E',
                              'C' => '352273',
                              'D' => '5DA55B',
                              'E' => 'B24F2F',
                              'F' => '36495A',
                              'G' => '759844',
                              'H' => '12203B',
                              'I' => '5D9645',
                              'J' => '5E5254',
                              'K' => '5F94A2',
                              'L' => '909C92',
                              'M' => 'AFC690',
                              'N' => '9D87B5',
                              'O' => '2AA370',
                              'P' => 'B68741',
                              'Q' => '8C91A5',
                              'R' => '29BF1F',
                              'S' => '885F7D',
                              'T' => '303BA4',
                              'U' => '68C55B',
                              'V' => '9A544C',
                              'W' => '764EB1',
                              'X' => '92BCB0',
                              'Y' => '651882',
                              'Z' => '579590',
                              '1' => 'AF3530',
                              '2' => '0E8423',
                              '3' => '9F811E',
                              '4' => '5E8BDC',
                              '5' => '68B7C4',
                              '6' => '8D27B1',
                              '7' => 'B2B6B7',
                              '8' => 'AF5D21',
                              '9' => '5D3D32',
                              '0' => '010207',
                              '-' => '6F7A80',
                              '=' => 'A459B4',
                              '!' => '27313A',
                              '@' => '177A65',
                              '#' => '030000',
                              '$' => '010002',
                              '%' => '414960',
                              '&' => '11C411',
                              '*' => '148083',
                              '(' => 'CBD773',
                              ')' => 'A230AC',
                              '_' => '2B594F',
                              '+' => 'F9F5F2',
                              '[' => '90C050',
                              '{' => '064056',
                              ']' => '35746C',
                              '}' => '5B110E',
                              ';' => '64828D',
                              ':' => '44466F',
                              '\'' => '190D95',
                              ',' => '768394',
                              '<' => '9D8770',
                              '.' => '3C4C66',
                              '>' => '3FB151',
                              '/' => '8B8F98',
                              '?' => '71948D',
                              '\\' => '6A617E',
                              '|' => '86745C'
                             );

    var $_brightness    = null;
    var $_usebase       = true;

    function setBrightness($degree)
    {
        // modify color brightness (-255/+255)

        $this->_brightness = $degree;
    }

    function setUsebase($usebase)
    {
        // if true (default) override colors 0-7 of the nsco console color
        // palette with the colors from the default quake3 console palette.

        $this->_usebase = $usebase;
    }

    function getColor($char)
    {
        // char to return NSCO color for
        // usebase overrides NSCO palette with default colors
        // brightness (positive or negative value to change color)
   
        $palette = $this->_NSCO_Colors;
   
        if ($this->_usebase) {
            foreach ($this->_BASE_Colors as $key => $value) {
                $palette[$key] = $value;
            }
        }
    
        if (isset($palette[$char])) {
            $color = $palette[$char];
        } else {
            // white
            $color = $palette['7'];
        }
    
        if (is_null($this->_brightness)) {
            return $color;
        } else {
            return $this->brightness($color, $this->_brightness);
        }
    }

    // the following two are taken and modified from PEAR::Image_Color

    function _splitcolor($hexcolor)
    {
        // split a hex color into its rgb parts

        $color[] = hexdec(substr($hexcolor, 0, 2));
        $color[] = hexdec(substr($hexcolor, 2, 2));
        $color[] = hexdec(substr($hexcolor, 4, 2));
        return $color;
    }
    
    function brightness($hexcolor, $degree = 10)
    {
        // use a positive value for brighter, negative for darker
        
        $color = $this->_splitcolor($hexcolor);

        for ($x = 0; $x < 3; $x++) {
            if (($color[$x] + $degree) < 256 ) {
                if (($color[$x] + $degree) > -1) {
                    $color[$x] += $degree;
                } else {
                    $color[$x] = 0;
                }
            } else {
                $color[$x] = 255;
            }
        }
        return sprintf('%02X%02X%02X', $color[0], $color[1], $color[2]);
    }

    function _addColorStart(&$colorarray, $color)
    {
        $colorarray[] = array('type' => 'color', 'data' => $color);
    }

    function _addColorEnd(&$colorarray)
    {
        $colorarray[] = array('type' => 'nocolor');
    }

    function _addChar(&$colorarray, $char)
    {
        $colorarray[] = array('type' => 'char', 'data' => $char);
    }

    function colorTextToArray($text)
    {
        // convert a quake3/nsco text string with color formatting
        // to an array we can process and format to whatever we want.

        // always start out with white
        $color = $oldcolor = $this->getColor('7');
        $state   = 0;
        $colorarray = array(); 
        $textseen = 0;
           
        $len = strlen($text);

        for ($i = 0; $i < $len; $i++) {
            $char = $text{$i};
            if ($state == 1 && ($char != '^')) {
                // is regular color description
                $color = $this->getColor($char);
                $state = 0;
            } elseif ($char == '^') {
                /// next char is color description
                $state = 1;
            } else {
                // is regular char, no intepretation
                if ($textseen == 0) {
                    $this->_addColorStart($colorarray, $color);
                } elseif ($color != $oldcolor) {
                    $this->_addColorEnd($colorarray);
                    $this->_addColorStart($colorarray, $color);
                } elseif (($i-1) == $len) {
                    $this->_addColorEnd($colorarray);

                }
                $this->_addChar($colorarray, $char);
                $oldcolor = $color;
                $textseen++;
            }
        }
        $this->_addColorEnd($colorarray);
        return $colorarray; 
    }

    function colorTextToText($text)
    {
        // return a text string with no color formatting

        $textstring = '';
        foreach ($this->colorTextToArray($text) as $item) {
            if ($item['type'] == 'char') {
                $textstring .= $item['data'];
            }
        }
        return $textstring;
    }

    function colorTextToHtml($text)
    {
        // return a text string with html color formatting

        $htmlstring = '';
        foreach ($this->colorTextToArray($text) as $item) {
            switch ($item['type']) {
                case 'color':
                    $htmlstring .= '<font color="#' . $item['data'] . '">';
                    break;
                case 'nocolor':
                    $htmlstring .= '</font>';
                    break;
                case 'char':
                    $htmlstring .= htmlspecialchars($item['data'], ENT_QUOTES);
            }
        }
        return $htmlstring;
    }

    function _getMircColor($color)
    {
        // given a hex color value return the corresponding mIRC color code

        if (isset($this->_MIRC_Colors[$color])) {
            return $this->_MIRC_Colors[$color];
        } else {
            // if no match approximate towards mIRC palette :)
            $lowest = 10000000;
            $color1 = $this->_splitcolor($color);
            foreach ($this->_MIRC_Colors as $hexcolor => $mirccolor) {
                $color2 = $this->_splitcolor($hexcolor);
                $r = ($color1[0] - $color2[0]) * ($color1[0] - $color2[0]);
                $g = ($color1[1] - $color2[1]) * ($color1[1] - $color2[1]);
                $b = ($color1[2] - $color2[2]) * ($color1[2] - $color2[2]);
                $delta = $r + $g + $b;
                if ($delta < $lowest) {
                    $mapcolor = $mirccolor;
                    $lowest = $delta;
                }
            }
            return $mapcolor;
        }
    } 

    function colorTextToIrc($text)
    {
        // return a text string with color formatting for output on IRC

        $ircstring = '';
        foreach ($this->colorTextToArray($text) as $item) {
            switch ($item['type']) {
                case 'color':
                    $ircstring .= "\003" 
                               . $this->_getMircColor($item['data'])
                               . ',01'; // force black background
                    break;
                case 'char':
                    $ircstring .= $item['data'];
            }
        }
        // terminate all colors at the end 
        return $ircstring . "\017";
    }
}
?>
